@@ -36,11 +36,10 @@ rb_thread_call_without_gvl(
 #endif /* ! HAVE_RB_THREAD_BLOCKING_REGION */
 #endif /* ! HAVE_RB_THREAD_CALL_WITHOUT_GVL */
 
-void init_mysql2_client();
-
 typedef struct {
   VALUE encoding;
   VALUE active_thread; /* rb_thread_current() or Qnil */
+  long server_version;
   int reconnect_enabled;
   int active;
   int connected;
@@ -50,4 +49,7 @@ typedef struct {
   MYSQL *client;
 } mysql_client_wrapper;
 
+void init_mysql2_client();
+void decr_mysql2_client(mysql_client_wrapper *wrapper);
+
 #endif
